<?php

namespace TDevAgency\Cms\Providers;

use DaveJamesMiller\Breadcrumbs\Facades\Breadcrumbs;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Blade;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Request;
use Illuminate\Support\ServiceProvider;
use TDevAgency\Cms\Commands\AddUser;
use TDevAgency\Cms\Commands\CmsInstall;
use TDevAgency\Cms\Commands\PublicAssets;
use TDevAgency\Cms\Models\BaseModels\AdminUser;
use TDevAgency\Cms\Models\BaseModels\AdminUserRole;
use TDevAgency\Cms\Observers\AdminUserObserver;
use TDevAgency\Cms\Observers\AdminUserRoleObserver;
use TDevAgency\Cms\Traits\ServiceProviderHelper;

class CmsServiceProvider extends ServiceProvider
{
    use ServiceProviderHelper;

    /**
     * @var string
     */
    private string $migrationsPath;
    /**
     * @var string
     */
    private string $noEditableTranslationsPath;
    /**
     * @var string
     */
    private string $translationsPath;
    /**
     * var string
     */

    private string $viewsPath;

    public function __construct($app)
    {
        parent::__construct($app);

        $this->noEditableTranslationsPath = TDEV_AGENCY_CMS_ROOT . 'resources' . DIRECTORY_SEPARATOR . 'noneditable-lang';
        $this->translationsPath           = TDEV_AGENCY_CMS_ROOT . 'resources' . DIRECTORY_SEPARATOR . 'lang';
        $this->viewsPath                  = TDEV_AGENCY_CMS_ROOT . 'resources' . DIRECTORY_SEPARATOR . 'views';
        $this->migrationsPath             = TDEV_AGENCY_CMS_ROOT . 'database' . DIRECTORY_SEPARATOR . 'migrations';
    }

    /**
     * Check is admin area
     */
    private function isAdmin(): bool
    {
        return preg_match('/\/' . $this->app['config']->get('cms.routes.admin_prefix') . '\//', Request::getUri()) ||
            preg_match('/\/' . $this->app['config']->get('cms.routes.admin_prefix') . '+$/', Request::getUri());
    }

    /**
     * Bootstrap services.
     *
     * @return void
     */
    public function boot()
    {
        AdminUser::observe(AdminUserObserver::class);
        AdminUserRole::observe(AdminUserRoleObserver::class);

        $this->publishes(
            [
                TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'cms.php'                   => config_path(
                    'cms.php'
                ),
                TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'cms_admin_sidebar_tpl.php' => config_path(
                    'cms_admin_sidebar.php'
                ),
            ],
            'config'
        );

        $this->publishes([
                             $this->translationsPath => resource_path('lang/vendor/cms'),
                         ], 'lang');

        $this->publishes([
                             $this->viewsPath => resource_path('views/vendor/cms'),
                         ], 'views');

        $this->loadMigrationsFrom($this->migrationsPath);
        $this->loadViewsFrom($this->viewsPath, 'cms');
        $this->loadTranslationsFrom($this->translationsPath, 'cms');
        $this->loadTranslationsFrom($this->noEditableTranslationsPath, 'cms-fixed');


        if (Config::get('app.debug')) {
            Blade::directive('xdebug', function ($expression) {
                return "<?php xdebug_break(); ?>";
            });
        }


        if ($this->app->runningInConsole()) {
            // Add commands here
            $this->commands([
                                CmsInstall::class,
                                AddUser::class,
                                PublicAssets::class,
                            ]);
        } else {
            if (class_exists('Breadcrumbs')) {
                require TDEV_AGENCY_CMS_ROOT . 'routes' . DIRECTORY_SEPARATOR . 'breadcrumbs.php';
                if (File::exists(base_path('routes/admin_breadcrumbs.php'))) {
                    require base_path('routes/admin_breadcrumbs.php');
                }
            }

            Breadcrumbs::macro('pageTitle', function () {
                $title = ($breadcrumb = Breadcrumbs::current()) ? (string)$breadcrumb->title : '';
                //TODO crash in edit page
                //if (($page = (int)request('page')) > 1) {
                //    $title .= ' - ' . trans('cms::breadcrumbs.page_no', ['page_no' => $page]);
                //}

                return $title;
            });

            Breadcrumbs::macro('pageMetaTitle', function () {
                $title = ($breadcrumb = Breadcrumbs::current()) ? (string)$breadcrumb->title . ' |' : '';
                if (($page = (int)request('page')) > 1) {
                    $title = sprintf('%s | %s', trans('cms::breadcrumbs.page_no', ['page_no' => $page]), $title);
                }

                return $title . ' | ' . Config::get('app.app_name');
            });
        }
    }

    /**
     * Register services.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfigFrom(
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'auth.php',
            'auth'
        );
        $this->mergeConfigFrom(
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'scout.php',
            'scout'
        );

        if (Config::get('app.debug')) {
            $this->mergeConfigFrom(
                TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'debugbar.php',
                'debugbar'
            );
        }

        $this->mergeConfigFrom(
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'cms.php',
            'cms'
        );

        if ($this->isAdmin()) {
            $this->mergeConfigFrom(
                TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'cms_admin_sidebar.php',
                'cms_admin_sidebar'
            );
            $this->mergeConfigFrom(
                TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'columnsortable.php',
                'columnsortable'
            );
            $this->mergeConfigFrom(
                TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'breadcrumbs.php',
                'breadcrumbs'
            );
        }

        if ($this->isAdmin()) {
            Config::set('app.locale', Config::get('cms.admin_locale'));
        }
    }

}
